package pushover

import (
	"context"
	"fmt"
	"log/slog"

	"github.com/go-resty/resty/v2"

	"github.com/usual2970/certimate/internal/pkg/core/notifier"
)

type NotifierConfig struct {
	// Pushover API Token。
	Token string `json:"token"`
	// 用户或分组标识。
	User string `json:"user"`
}

type NotifierProvider struct {
	config     *NotifierConfig
	logger     *slog.Logger
	httpClient *resty.Client
}

var _ notifier.Notifier = (*NotifierProvider)(nil)

func NewNotifier(config *NotifierConfig) (*NotifierProvider, error) {
	if config == nil {
		panic("config is nil")
	}

	client := resty.New()

	return &NotifierProvider{
		config:     config,
		logger:     slog.Default(),
		httpClient: client,
	}, nil
}

func (n *NotifierProvider) WithLogger(logger *slog.Logger) notifier.Notifier {
	if logger == nil {
		n.logger = slog.Default()
	} else {
		n.logger = logger
	}
	return n
}

func (n *NotifierProvider) Notify(ctx context.Context, subject string, message string) (res *notifier.NotifyResult, err error) {
	// REF: https://pushover.net/api
	req := n.httpClient.R().
		SetHeader("Content-Type", "application/json").
		SetBody(map[string]any{
			"title":   subject,
			"message": message,
			"token":   n.config.Token,
			"user":    n.config.User,
		})
	resp, err := req.Post("https://api.pushover.net/1/messages.json")
	if err != nil {
		return nil, fmt.Errorf("pushover api error: failed to send request: %w", err)
	} else if resp.IsError() {
		return nil, fmt.Errorf("pushover api error: unexpected status code: %d, resp: %s", resp.StatusCode(), resp.String())
	}

	return &notifier.NotifyResult{}, nil
}
